@@ -5,37 +5,68 @@ from base64 import urlsafe_b64decode
 import requests
 from requests.exceptions import HTTPError
 from flask import Request, jsonify
-from . import util
+from google.cloud import tasks
 
 
 SERVICE_CATALOG = json.loads(os.environ["SERVICE_CATALOG"])
+QUEUE_CATALOG = json.loads(os.environ["QUEUE_CATALOG"])
+TASKS = tasks.CloudTasksClient()
 
 
-def _get_auth_token(audience):
-    token_url = (
+def get_service_account_metadata(field, params=None):
+    url = (
         "http://metadata.google.internal"
-        "/computeMetadata/v1/instance/service-accounts/default/identity"
-        f"?audience={audience}"
+        "/computeMetadata/v1/instance/service-accounts/default/{field}"
     )
     return requests.get(
-        url=token_url,
-        headers={"Metadata-Flavor": "Google"},
+        url=url, headers={"Metadata-Flavor": "Google"}, params=params
     ).content.decode()
 
 
-def call(service_name, service_path=None, **kwargs):
+def get_service_url(service_name, service_path=None):
     url = SERVICE_CATALOG.get(service_name)
     if not url:
         raise Exception(f"Service '{service_name}' not found in catalog.")
     if service_path:
         url += f"/{service_path}"
-    auth = {"Authorization": "Bearer " + _get_auth_token(url)}
+    return url
+
+
+def call(service_name, service_path=None, **kwargs):
+    url = get_service_url(service_name, service_path)
+    auth = {
+        "Authorization": "Bearer "
+        + get_service_account_metadata("identity", {"audience": url})
+    }
     r = requests.post(url, headers=auth, json=kwargs)
     if r.status_code >= 500:
         r.raise_for_status()
     return r
 
 
+def queue(calls):
+    scheduled = []
+    for call in calls:
+        service_name = call["service_name"]
+        service_path = call["service_path"]
+        data = call["data"]
+        task = TASKS.create_task(
+            parent=QUEUE_CATALOG[service_name],
+            task=tasks.Task(
+                http_request=tasks.HttpRequest(
+                    http_method=tasks.HttpMethod.POST,
+                    url=get_service_url(service_name, service_path),
+                    body=json.dumps(data).encode(),
+                    oidc_token=tasks.OidcToken(
+                        service_account_email=get_service_account_metadata("email")
+                    ),
+                )
+            ),
+        )
+        scheduled.append(task)
+    return scheduled
+
+
 def user_info(request: Request):
     encoded_user_info = request.headers.get("X-Endpoint-Api-Userinfo")
     if encoded_user_info:
@@ -69,7 +100,7 @@ def validate(**kwargs):
                     valid = kwargs[key](request.json.get(key))
                     if not valid:
                         return (
-                            jsonify(message=f"Invalid parameter value for '{key}'"),
+                            jsonify(message=f"Invalid value for '{key}'"),
                             400,
                         )
             except Exception as e:
